/* {{{
 * CalcRogue, a roguelike game for PCs, calculators and PDAs
 * Copyright (C) 2003 Jim Babcock
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * }}} */
// options.c
/// @file
/// @brief Player-configurable options menu

#include "crogue.h"
#include "export.h"

/// A configuration option
typedef struct option
{
	const char *description;
	const char **choices;
	uchar default_value;
} option;

#ifndef PALMOS
#	define DEFAULT_AUTOPICKUP OPTION_PROMPT
#	define DEFAULT_AUTOFOLLOW OPTION_PROMPT
#else
#	define DEFAULT_AUTOPICKUP OPTION_NO
#	define DEFAULT_AUTOFOLLOW OPTION_NO
#endif

//{{{
static const char *choices_yesnoprompt[] = {
	gettext("Yes"),
	gettext("No"),
	gettext("Prompt"),
	NULL };
//}}}
//{{{
static const char *choices_yesno[] = {
	gettext("Yes"),
	gettext("No"),
	NULL };
//}}}
#ifdef IS_CALCULATOR
//{{{
static const char *choices_smalllarge[] = {
	gettext("Small"),
	gettext("Large"),
	NULL };
//}}}
//{{{
static const char *choices_fontsize[] = {
	gettext("Small"),
	gettext("Medium"),
	gettext("Large"),
	NULL };
//}}}
#endif

//{{{
static const option options[] = {
		{ gettext("Auto-pickup"),
		  choices_yesnoprompt,
		  DEFAULT_AUTOPICKUP
		},
		{ gettext("Auto-follow-stairs"),
		  choices_yesnoprompt,
		  DEFAULT_AUTOFOLLOW
		},
#ifdef PALMOS
		{ gettext("Use dialog for prompts"),
		  choices_yesno,
		  OPTION_YES
		},
#endif
#ifdef IS_CALCULATOR
		{ gettext("Tile size"),
		  choices_smalllarge,
		  OPTION_SIZE_SMALL
		},
		{ gettext("Font size"),
		  choices_fontsize,
		  OPTION_FONT_SMALL
		},
		{ gettext("Archive saves"),
		  choices_yesno,
		  OPTION_NO
		},
		{ gettext("Archive scores"),
		  choices_yesno,
		  OPTION_YES
		},
		{ gettext("Use grayscale"),
		  choices_yesno,
		  OPTION_YES
		},
#endif
		{ NULL },
	};
//}}}
//{{{
static void cb_options_menu(int n, char *buf)
{
	sprintf(buf, "%s\t%s", options[n].description, options[n].choices[w->options[n]]);
}
//}}}

//{{{
int count_options(void)
{
	int ii;
	for(ii=0; ; ii++)
	{
		if(!options[ii].description)
			return ii;
	}
}
//}}}
//{{{
int count_choices(const char **choices)
{
	int ret=0;
	while(*choices != NULL) {
		choices++;
		ret++;
	}
	return ret;
}
//}}}
//{{{
void init_options(void)
{
	int ii;
	int num_opts = count_options();
	for(ii=0; ii<num_opts; ii++)
		w->options[ii] = options[ii].default_value;
}
//}}}
//{{{
void options_menu(void)
{
	int choice;
	setTabStops(options_menu_tabs);
	UI_Menu_Set_Persist(1);
	while(1) {
		choice = UI_Menu_Pick(options_rect, count_options(), &cb_options_menu);
		if(choice<0)
			break;
		w->options[choice] ++;
		w->options[choice] %= count_choices(options[choice].choices);
		
#ifdef IS_CALCULATOR
		if(choice == OPTION_GRAYSCALE)
		{
			if(w->options[OPTION_GRAYSCALE] == OPTION_GRAY_ON)
				GrayOnThrow();
			else
				GrayOff();
			if(w->level) // Only if game is started
				full_redraw();
		}
#endif
	}
	UI_Menu_Set_Persist(0);
}
//}}}
