#include "crogue.h"

static void init_file_prefix(const char *prefix);
static void handle_args(int argc, char **argv);

//{{{
int main(int argc, char **argv)
{
	init_file_prefix(argv[0]);
	handle_args(argc, argv);
	initialize();
	eventloop();
	return 0;
}
//}}}
char *file_prefix;
//{{{
static void init_file_prefix(const char *prefix)
{
	int ii, jj;
	
	for(ii=strlen(prefix)-1; ii>=0; ii--) {
		if(prefix[ii] == '/' || prefix[ii] == '\\')
			break;
	}
	
	file_prefix = (char*)malloc_throw(ii+1);
	for(jj=0; jj<=ii; jj++)
		file_prefix[jj] = prefix[jj];
	file_prefix[jj] = '\0';
}
//}}}

//{{{
static void handle_args(int argc, char **argv)
{
//	int ii;
//	for(ii=1; ii<argc; ii++)
//	{
//		if(!strcmp(argv[ii], "-h")) {
//			printf("Usage: %s [-L language]\n", argv[0]);
//			exit(0);
//		}
//		if(!strcmp(argv[ii], "-L")) {
//			ii++;
//			if(ii>=argc) {
//				fprintf(stderr, "You must specify a language file.\n");
//				exit(1);
//			}
//			if( gettext_open(argv[ii]) == 0 ) // Error opening
//			{
//				fprintf(stderr, "Error opening language file.\n"
//				                "Press ENTER to continue\n");
//				getchar();
//				fflush(stdin);
//			}
//		}
//	}
}
//}}}

//{{{
const char *asciistart[24] =
	{
" +--------------------------------------------------------------------------+   ",
" |                ########                             #####                |   ",
" |              #######                                  #####              |   ",
" |            #######             ###############         #####             |   ",
" |           ######           ######################       #####            |   ",
" |           #####          ########  # #   #  ######       #####           |   ",
" |          ######        ####### CalcRogue ###########      #####          |   ",
" |          #####        #######    by Jim Babcock #####      ####          |   ",
" |         ######        ######    ####+##  #############     #####         |   ",
" |         ######       ######     #.....#  #       #####     #####         |   ",
" |         ######       ############..V.T#  #  }    #####     #####         |   ",
" |         ######       ######     +.D...#  #       ######    #####         |   ",
" |         ######       ############.W..@#  #       #####     #####         |   ",
" |          ######      ######     #####-####+##########     #####          |   ",
" |          ######       ######    #   ...     +   ######   ######          |   ",
" |           ######       ######   ##############################           |   ",
" |            ######       ########              ##############             |   ",
" |             ######         ######################  ########              |   ",
" |              #######         ##################      ####                |   ",
" |                ########         ###########                  ###         |   ",
" |                   ########                                 #####         |   ",
" |                     ##########                          #######          |   ",
" +--------------------------------------------------------------------------+   ",
"                                                                                "
	};
//}}}
//{{{
const char *asciicolors[24] = {
" BBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBB   ",
" B                ########                             #####                B   ",
" B              #######                                  #####              B   ",
" B            #######             ###############         #####             B   ",
" B           ######           ######################       #####            B   ",
" B           #####          ########uuuuuuuuuuu######       #####           B   ",
" B          ######        #######uWWWWWWWWWuuuuuu######      #####          B   ",
" B          #####        #######uuuuWWWWWWWWWWWWWWu#####      ####          B   ",
" B         ######        ######uuuuuuuuuuuuuuuuuuuuu#####     #####         B   ",
" B         ######       ######uuuuuuuuuuuuuuuuuuuuuu#####     #####         B   ",
" B         ######       ######uuuuuuuuuuuuuuuuuuuuuuu####     #####         B   ",
" B         ######       ######uuuuuuuuuuuuuuuuuuuuuu######    #####         B   ",
" B         ######       ######uuuuuuuuuuuuuuuuuuuuuu#####     #####         B   ",
" B          ######      ######uuuuuuuuuuuuuuuuuuuuuu####     #####          B   ",
" B          ######       ######uuuuuuuuuuuuuuuuuuuu######   ######          B   ",
" B           ######       ######uuuuuuuuuuuuuuuuuu###############           B   ",
" B            ######       ########uuuuuuuuuuuuuu##############             B   ",
" B             ######         ######################  ########              B   ",
" B              #######         ##################      ####                B   ",
" B                ########         ###########                  ###         B   ",
" B                   ########                                 #####         B   ",
" B                     ##########                          #######          B   ",
" BBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBB   ",
"                                                                                "
	};
//}}}

//{{{
void display_title_screen(void)
{
	int ii, jj, color;
	for(ii=0; ii<24; ii++)
	for(jj=0; jj<80; jj++)
	{
		switch(toupper(asciicolors[ii][jj])) {
			case 'Y': color = YELLOW; break;
			case 'U': color = BLUE;   break;
			case 'R': color = RED;    break;
			case 'B': color = BLACK;  break;
			default:  color = WHITE;  break;
		}
		attroff(COLOR_PAIR(DEFAULT_COLOR));
		if(isupper(asciicolors[ii][jj]))
			attron(A_BOLD);
		attron(COLOR_PAIR(color+1));
		mvaddch(ii, jj, asciistart[ii][jj]);
		attroff(COLOR_PAIR(color+1));
		if(isupper(asciicolors[ii][jj]))
			attroff(A_BOLD);
	}
	attron(A_BOLD);
	mvaddstr(20, 50, CROGUE_VERSION);
	attroff(A_BOLD);
}
//}}}
//{{{
void init_layout(void)
{
	uint tilefield_top, tilefield_bottom;
	uint tilefield_height;
	uint tilefield_width;
	uint status_pos;
	const char *tileset;
	
	tileset = tiles;
	
	status_pos = SCREEN_HEIGHT - 2;
	tilefield_top = 2;
	tilefield_bottom = status_pos - 1;
	tilefield_height = tilefield_bottom-tilefield_top;
	
	tilefield_width = SCREEN_WIDTH;
	
	UI_MF_Place(0, 2);
	UI_Status_Place(status_pos, 2);
	
	UI_TF_Place(
			COORD(0, tilefield_top),
			COORD(tilefield_width, tilefield_height),
			tileset, 1, 1
			);
}
//}}}

//{{{
void sys_init(void)
{
	(void)initscr();
	if(has_colors())
	{
		if(start_color() == ERR)
			panic("Error initializing color.");
		init_pair(1, COLOR_BLACK,   COLOR_BLACK);
		init_pair(2, COLOR_RED,     COLOR_BLACK);
		init_pair(3, COLOR_GREEN,   COLOR_BLACK);
		init_pair(4, COLOR_YELLOW,  COLOR_BLACK);
		init_pair(5, COLOR_BLUE,    COLOR_BLACK);
		init_pair(6, COLOR_MAGENTA, COLOR_BLACK);
		init_pair(7, COLOR_CYAN,    COLOR_BLACK);
		init_pair(8, COLOR_WHITE,   COLOR_BLACK);
	}
	if(attron(COLOR_PAIR(DEFAULT_COLOR)) == ERR)
		panic("attron() returned ERR");
	if(attroff(A_BOLD) == ERR)
		panic("attroff() returned ERR");
	noecho();
	cbreak();
	if(notimeout(stdscr, TRUE) == ERR)
		panic("notimeout() returned ERR");
	if(keypad(stdscr, TRUE) == ERR)
		panic("keypad() returned ERR");
#	ifdef USE_NCURSES
		ESCDELAY=0;
#	endif
}
//}}}
//{{{
void sys_init_data_file(void)
{
	FILE *constfile;
	
	// Load constant-data file
	constfile = fopen(retprintf("%s%s", file_prefix, DATAFILENAME_PC), "rb");
	if(!constfile)
		panic("Couldn't open constant data file.");
	// This is much simpler than figuring out how large the file really is.
	// We know it can't be more than 64k, because then it wouldn't fit in
	// the calculator version. Of course this is wasting memory, but it's
	// not in any mem-critical environments.
	w->constfileoffset = debug_malloc(65536);
	(void)fread(w->constfileoffset, 65536, 1, constfile);
	memcpy(&w->desc, w->constfileoffset, sizeof(struct world_descriptor));
	fclose(constfile);
	
	link_data_file();
}
//}}}
//{{{
void sys_cleanup(void)
{
	if(w && w->constfileoffset)
		free(w->constfileoffset);
	clrscr();
	move(0, 0);
	endwin();
}
//}}}

//{{{
int curses_title_screen(void)
{
	int choice;
	clrscr();
	
	display_title_screen();
	
	do {
		choice = read_char();
	} while(choice < 'a' || choice > 'e');
	choice -= 'a';
	return choice;
}
//}}}

