#include <stdio.h>
#include "lir.h"
#include "util.h"

const int num_instructions = 54;

//{{{
argument_info single_register[] = { {1, 0}         };
argument_info double_register[] = { {1, 0}, {1, 0} };
argument_info immediate_byte[]  = { {1, 0}         };
argument_info immediate_word[]  = { {2, 0}         };
argument_info immediate_long[]  = { {4, 0}         };
argument_info immediate_call[]  = { {2, 1}         };
argument_info offset_byte[]     = { {1, 1}         };
argument_info offset_word[]     = { {2, 1}         };
argument_info offset_long[]     = { {4, 1}         };
//}}}
//{{{
instruction_info_t instructions[] = {
	{ 0x00, "HLT",    1,  0,  NULL,                 id_null },
	{ 0x01, "NOP",    1,  0,  NULL,                 id_null },
	{ 0x02, "DISC",   1,  0,  NULL,                 id_null },
	{ 0x03, "DISCN",  2,  1,  immediate_byte,       id_null },
	{ 0x04, "DUP",    1,  0,  NULL,                 id_null },
	{ 0x05, "ALLOCA", 1,  0,  NULL,                 id_null },
	{ 0x10, "RET",    1,  0,  NULL,                 id_null },
	{ 0x11, "CALL",   3,  0,  NULL,                 id_null },
	{ 0x12, "STA",    1,  0,  NULL,                 id_null },
	{ 0x13, "MVA",    2,  1,  single_register,      id_null },
	{ 0x14, "LINK",   2,  1,  immediate_byte,       id_null },
	{ 0x15, "CCALL",  3,  0,  NULL,                 id_null },
	{ 0x20, "LDR",    2,  1,  single_register,      id_null },
	{ 0x21, "LDR2",   3,  2,  double_register,      id_null },
	{ 0x22, "STR",    2,  1,  single_register,      id_null },
	{ 0x23, "MOV",    3,  2,  double_register,      id_null },
	{ 0x24, "LDIB",   2,  1,  immediate_byte,       id_null },
	{ 0x25, "LDIW",   3,  1,  immediate_word,       id_null },
	{ 0x26, "LDIL",   5,  1,  immediate_long,       id_null },
	{ 0x27, "LDIZ",   1,  0,  NULL,                 id_null },
	{ 0x28, "LDIO",   1,  0,  NULL,                 id_null },
	{ 0x30, "ADD",    1,  0,  NULL,                 id_null },
	{ 0x31, "SUB",    1,  0,  NULL,                 id_null },
	{ 0x32, "MUL",    1,  0,  NULL,                 id_null },
	{ 0x33, "DIV",    1,  0,  NULL,                 id_null },
	{ 0x34, "MOD",    1,  0,  NULL,                 id_null },
	{ 0x35, "ASL",    1,  0,  NULL,                 id_null },
	{ 0x36, "ASR",    1,  0,  NULL,                 id_null },
	{ 0x37, "AND",    1,  0,  NULL,                 id_null },
	{ 0x38, "OR",     1,  0,  NULL,                 id_null },
	{ 0x39, "XOR",    1,  0,  NULL,                 id_null },
	{ 0x3A, "NEG",    1,  0,  NULL,                 id_null },
	{ 0x3B, "INCR",   2,  0,  single_register,      id_null },
	{ 0x3C, "DECR",   2,  0,  single_register,      id_null },
	{ 0x40, "BOOL",   1,  0,  NULL,                 id_null },
	{ 0x41, "BOOL2",  1,  0,  NULL,                 id_null },
	{ 0x42, "NOT",    1,  0,  NULL,                 id_null },
	{ 0x43, "TEQ",    1,  0,  NULL,                 id_null },
	{ 0x44, "TLT",    1,  0,  NULL,                 id_null },
	{ 0x45, "TGT",    1,  0,  NULL,                 id_null },
	{ 0x50, "JMPB",   2,  1,  offset_byte,          id_null },
	{ 0x51, "JMPW",   3,  1,  offset_word,          id_null },
	{ 0x52, "JMPL",   5,  1,  offset_long,          id_null },
	{ 0x53, "BRAB",   2,  1,  offset_byte,          id_null },
	{ 0x54, "BRAW",   3,  1,  offset_word,          id_null },
	{ 0x55, "BRAL",   5,  1,  offset_long,          id_null },
	{ 0x60, "DEREFB", 1,  1,  NULL,                 id_null },
	{ 0x61, "DEREFW", 1,  1,  NULL,                 id_null },
	{ 0x62, "DEREFL", 1,  1,  NULL,                 id_null },
	{ 0x63, "POKEB",  1,  1,  NULL,                 id_null },
	{ 0x64, "POKEW",  1,  1,  NULL,                 id_null },
	{ 0x65, "POKEL",  1,  1,  NULL,                 id_null },
	{ 0x66, "LDLBL",  5,  1,  offset_long,          id_null }
	};
//}}}

lir_instr_t *lir_data = NULL;
int instr_num=0, instr_alloc=0;

static void lir_print_instruction(lir_instr_t *instruction);

//{{{
void lir_write_instruction(lir_instr_t instruction)
{
	expand_table((void**)&lir_data, &instr_alloc, instr_num+1, sizeof(lir_instr_t));
	lir_data[instr_num++] = instruction;
}
//}}}
//{{{
void lir_write_inst(int op)
{
	lir_instr_t addition;
	addition.type = op;
	lir_write_instruction(addition);
}
//}}}
//{{{
void lir_write_inst_reg(int op, int reg)
{
	lir_instr_t addition;
	addition.type = op;
	addition.args.reg = reg;
	lir_write_instruction(addition);
}
//}}}
//{{{
void lir_write_inst_2_reg(int op, int reg1, int reg2)
{
	reg_pair_t regpair;
	lir_instr_t addition;
	regpair.reg1 = reg1;
	regpair.reg2 = reg2;
	addition.type = op;
	addition.args.regpair = regpair;
	lir_write_instruction(addition);
}
//}}}
//{{{
void lir_write_inst_immediate(int op, long val)
{
	lir_instr_t addition;
	addition.type = op;
	addition.args.immed_val = val;
	lir_write_instruction(addition);
}
//}}}
//{{{
void lir_write_inst_label(int op, id_handle_t label)
{
	lir_instr_t addition;
	addition.type = op;
	addition.args.target = label;
	lir_write_instruction(addition);
}
//}}}
//{{{
void lir_write_label(id_handle_t identifier)
{
	lir_write_label_flags(identifier, CODE_LABEL_FLAGS, 0, 0);
}
//}}}
//{{{
void lir_write_label_flags(id_handle_t identifier, short flags, short forced_offset, char force_offset)
{
	lir_instr_t addition;
	addition.type = LIR_LABEL;
	addition.args.lbl.identifier = identifier;
	addition.args.lbl.flags = flags;
	addition.args.lbl.force_offset = force_offset;
	addition.args.lbl.forced_offset = forced_offset;
	lir_write_instruction(addition);
}
//}}}

//{{{
void lir_print(void)
{
	int i;
	
	for(i=0; i<instr_num; i++)
		lir_print_instruction(&lir_data[i]);
}
//}}}

//{{{
static void lir_print_instruction(lir_instr_t *instruction)
{
	if(instruction->type >= num_instructions)
	{
		switch(instruction->type)
		{
			case LIR_LABEL:
				printf("%s (%i):\n", get_identifier_string(instruction->args.lbl.identifier), (int)instruction->args.lbl.flags);
				break;
			case LIR_EVEN:
				printf("\tEVEN\n");
				break;
			case LIR_CHAR  : printf("\tCHAR %li\n",   instruction->args.immed_val); break;
			case LIR_UCHAR : printf("\tUCHAR %li\n",  instruction->args.immed_val); break;
			case LIR_SHORT : printf("\tSHORT %li\n",  instruction->args.immed_val); break;
			case LIR_USHORT: printf("\tUSHORT %li\n", instruction->args.immed_val); break;
			case LIR_LONG  : printf("\tLONG %li\n",   instruction->args.immed_val); break;
			case LIR_ULONG : printf("\tULONG %li\n",  instruction->args.immed_val); break;
			case LIR_FLINK : printf("\tFLINK %s\n",   get_identifier_string(instruction->args.target)); break;
			default:
				printf("INVALID INSTRUCTION\n");
				break;
		}
	}
	else
	{
		printf("\t%s", instructions[instruction->type].mnemonic);
		
		switch(instruction->type)
		{
			// Single immediate value
			case LIR_LDIB:
			case LIR_LDIW:
			case LIR_LDIL:
			case LIR_DISCN:
			case LIR_CCALL:
			case LIR_LINK:
				printf(" %li", instruction->args.immed_val);
				break;
			
			// Single register
			case LIR_MVA:
			case LIR_LDR:
			case LIR_STR:
			case LIR_INCR:
			case LIR_DECR:
				printf(" %i", instruction->args.reg);
				break;
			
			// Double register
			case LIR_LDR2:
			case LIR_MOV:
				printf(" %i %i", instruction->args.regpair.reg1, instruction->args.regpair.reg2);
				break;
			
			// Single label
			case LIR_CALL:
			case LIR_JMPB:
			case LIR_JMPW:
			case LIR_JMPL:
			case LIR_BRAB:
			case LIR_BRAW:
			case LIR_BRAL:
			case LIR_LDLBL:
				printf(" %s", get_identifier_string(instruction->args.target));
				break;
			
			// No args
			default:
				break;
		}
		
		printf("\n");
	}
}
//}}}

